/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.ext.ensembl;

import jalview.datamodel.AlignmentI;

import java.io.BufferedReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;
import java.util.List;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

/**
 * A client for the Ensembl lookup REST endpoint; used to find the Parent gene
 * identifier given a transcript identifier.
 * 
 * @author gmcarstairs
 *
 */
public class EnsemblLookup extends EnsemblRestClient
{
  private static final String SPECIES = "species";

  private static final String PARENT = "Parent";

  /**
   * Default constructor (to use rest.ensembl.org)
   */
  public EnsemblLookup()
  {
    super();
  }

  /**
   * Constructor given the target domain to fetch data from
   * 
   * @param
   */
  public EnsemblLookup(String d)
  {
    super(d);
  }

  @Override
  public String getDbName()
  {
    return "ENSEMBL";
  }

  @Override
  public AlignmentI getSequenceRecords(String queries) throws Exception
  {
    return null;
  }

  @Override
  protected URL getUrl(List<String> ids) throws MalformedURLException
  {
    String identifier = ids.get(0);
    return getUrl(identifier);
  }

  /**
   * @param identifier
   * @return
   */
  protected URL getUrl(String identifier)
  {
    String url = getDomain() + "/lookup/id/" + identifier
            + "?content-type=application/json";
    try
    {
      return new URL(url);
    } catch (MalformedURLException e)
    {
      return null;
    }
  }

  @Override
  protected boolean useGetRequest()
  {
    return true;
  }

  @Override
  protected String getRequestMimeType(boolean multipleIds)
  {
    return "application/json";
  }

  @Override
  protected String getResponseMimeType()
  {
    return "application/json";
  }

  /**
   * Calls the Ensembl lookup REST endpoint and retrieves the 'Parent' for the
   * given identifier, or null if not found
   * 
   * @param identifier
   * @return
   */
  public String getParent(String identifier)
  {
    return getAttribute(identifier, PARENT);
  }

  /**
   * Calls the Ensembl lookup REST endpoint and retrieves the 'species' for the
   * given identifier, or null if not found
   * 
   * @param identifier
   * @return
   */
  public String getSpecies(String identifier)
  {
    return getAttribute(identifier, SPECIES);
  }

  /**
   * @param identifier
   * @param attribute
   * @return
   */
  protected String getAttribute(String identifier, String attribute)
  {
    List<String> ids = Arrays.asList(new String[] { identifier });

    BufferedReader br = null;
    try
    {
      URL url = getUrl(identifier);
      if (url != null)
      {
        br = getHttpResponse(url, ids);
      }
      return (parseResponse(br, attribute));
    } catch (IOException e)
    {
      // ignore
      return null;
    } finally
    {
      if (br != null)
      {
        try
        {
          br.close();
        } catch (IOException e)
        {
          // ignore
        }
      }
    }
  }

  /**
   * Parses the value of 'attribute' from the JSON response and returns the
   * value, or null if not found
   * 
   * @param br
   * @param attribute
   * @return
   * @throws IOException
   */
  protected String parseResponse(BufferedReader br, String attribute) throws IOException
  {
    String parent = null;
    JSONParser jp = new JSONParser();
    try
    {
      JSONObject val = (JSONObject) jp.parse(br);
      parent = val.get(attribute).toString();
    } catch (ParseException | NullPointerException e)
    {
      // ignore
    }
    return parent;
  }

}
